@@ -1,7 +1,8 @@
 #!/bin/bash
-VERSION='0.3.4'
+VERSION='0.4.0'
 #
-# Copyright (C) 2003-2006  Ola Lundqvist <opal@debian.org>
+# Copyright (C) 2003-2007  Ola Lundqvist <opal@debian.org>
+# Copyright (C) 2007       Jochen Voss <voss@seehuhn.de>
 # Copyright (C) 2006       Miroslav Kure <kurem@upcase.inf.upol.cz>
 # Copyright (C) 2002       Mark Lawrence <nomad@null.net>
 # Copyright (C) 2002       Paul Sladen   <vserver@paul.sladen.org>
@@ -65,6 +66,12 @@ DIST="etch"
 # Local or nearest location of a debian mirror (must include the `/debian')
 MIRROR="http://ftp.uk.debian.org/debian"
 
+# debian-non-US mirror (must include the '/debian-non-US', only used for woody)
+MIRROR_NON_US="http://non-us.debian.org/debian-non-US"
+
+# mirror for security updates
+MIRROR_SECURITY="http://security.debian.org"
+
 # Default network interface for vservers:
 INTERFACE="eth0"
 
@@ -72,13 +79,19 @@ if [ -r /etc/vservers/util-vserver-vars ] ; then
     . /etc/vservers/util-vserver-vars
 fi
 
+# The name of a debconf database to be read by debconf's file driver.
+# This can be used to store the answers for debconf questions during
+# install and thus allows for unattended installation.  See the
+# debconf.conf(5) manual page for the file format.
+DEBCONF_FILE_DB=""
+
 if [ -r /etc/vservers/newvserver-vars ] ; then
     . /etc/vservers/newvserver-vars    
 fi
 
 # Support for 0.30.203 and above of util-vserver
 if [ -n "$DEFAULT_VSERVERDIR" ] ; then
-    VROOTDIR = "$DEFAULT_VSERVERDIR"
+    VROOTDIR="$DEFAULT_VSERVERDIR"
 fi
 
 # ----------------------------------------------------------------
@@ -120,6 +133,13 @@ Required:
 	--hostname		hostname for new vserver (eg. "alpha")
 	--domain		dns domain for new vserver (eg. "example.com")
 	--ip			IPv4 address for new vserver
+				(syntax: --ip <ip>[/<netmask|prefixlength>])
+
+	Either --ip or --domain/--hostname may be omitted if the
+	corresponding information can be found via a name server
+	query. For this to work the dnsutils package must be installed.
+
+You can also set variables in /etc/vservers/newvserver-vars.
 
 EOF
 }
@@ -127,9 +147,11 @@ full_version ()
 {
     cat << EOF
 ${0##*/} version $VERSION
-Copyright (c) 2002 Mark Lawrence <nomad@null.net>
-Copyright (c) 2002 Paul Sladen   <vserver@paul.sladen.org>
-Copyright (c) 2003 Ola Lundqvist <opal@debian.org>
+Copyright (C) 2003-2007  Ola Lundqvist <opal@debian.org>
+Copyright (C) 2007       Jochen Voss <voss@seehuhn.de>
+Copyright (C) 2006       Miroslav Kure <kurem@upcase.inf.upol.cz>
+Copyright (C) 2002       Mark Lawrence <nomad@null.net>
+Copyright (C) 2002       Paul Sladen   <vserver@paul.sladen.org>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -316,14 +338,45 @@ parse_args ()
 
 parse_args $@
 
+if [ -x "$(which dig)" ] ; then
+    if [ -n "$IP" -a \( -z "$VHOST" -o -z "$VDOMAIN" \) ]; then
+	tmp=$(dig +short -x $(echo $IP | sed 's;/[0-9]*$;;'))
+	if [ -n "$tmp" ]; then
+	    if [ -z "$VHOST" ]; then
+		VHOST=$(echo $tmp | sed 's/\..*//')
+	    fi
+	    if [ -z "$VDOMAIN" ]; then
+		VDOMAIN=$(echo $tmp | sed 's/[^.]*\.//')
+	    fi
+	fi
+    fi
+    
+    if [ -n "$VHOST" -a -n "$VDOMAIN" -a -n "$INTERFACE" -a -z "$IP" ]; then
+	prefix=$(ip -o addr show dev "$INTERFACE" primary scope global | \
+	    sed -n 's;.*inet [0-9.]*/\([0-9]*\).*;\1;p')
+	IP="$(dig +short "$VHOST.$VDOMAIN")/$prefix"
+    fi
+    
+    if ! [ -n "$VHOST" -a -n "$VDOMAIN" -a -n "$IP" ]; then
+	echo "${0##*/} error: --hostname, --domain and --ip are required" 1>&2
+	usage
+	exit 1
+    fi
+fi
+
+
 if ! [ -n "$VHOST" -a -n "$VDOMAIN" -a -n "$IP" -a -n "$NFS_SERVER" ]; then
     echo "${0##*/} error: --hostname, --domain, --ip and --nfs-server are required" 1>&2
     usage
     exit 1
 fi
 
+IP_ADDR=${IP%%/*}
+
 # Strip final slashes off a couple of things
 MIRROR="${MIRROR%/}"
+MIRROR_NON_US="${MIRROR_NON_US%/}"
+MIRROR_SECURITY="${MIRROR_SECURITY%/}"
 VROOTDIR="${VROOTDIR%/}"
 
 # Check we've got debootstrap available
@@ -403,10 +456,10 @@ if [ "woody" == "$DIST" ] ; then
 deb $MIRROR/ $DIST main
 deb-src $MIRROR/ $DIST main
 
-deb http://non-us.debian.org/debian-non-US $DIST/non-US main
-deb-src http://non-us.debian.org/debian-non-US $DIST/non-US main
+deb $MIRROR_NON_US $DIST/non-US main
+deb-src $MIRROR_NON_US $DIST/non-US main
 
-deb http://security.debian.org $DIST/updates main
+deb $MIRROR_SECURITY $DIST/updates main
 
 EOF
 else
@@ -414,7 +467,7 @@ else
 deb $MIRROR/ $DIST main
 deb-src $MIRROR/ $DIST main
 
-deb http://security.debian.org $DIST/updates main
+deb $MIRROR_SECURITY $DIST/updates main
 
 EOF
 fi
@@ -437,7 +490,7 @@ cat << EOF > "$VROOTDIR/$VHOST/etc/hosts"
 # /etc/hosts
 
 127.0.0.1	localhost
-$IP	$VHOST.$VDOMAIN $VHOST
+$IP_ADDR	$VHOST.$VDOMAIN $VHOST
 
 # The following lines are desirable for IPv6 capable hosts
 # (added automatically by netbase upgrade)
@@ -464,7 +517,10 @@ iface lo inet loopback
 EOF
 
 # grab DNS servers from the host-server for `resolv.conf'
-(echo search $VDOMAIN ; grep '^nameserver' /etc/resolv.conf) \
+HOST_IP=$(ip -o addr show dev $INTERFACE primary scope global | \
+          sed -n 's;.*inet \([^/]*\)/.*;\1;p')
+(echo search $VDOMAIN;
+ grep '^nameserver' /etc/resolv.conf | sed "s/127.0.0.1/$HOST_IP/") \
     > "$VROOTDIR/$VHOST/etc/resolv.conf"
 
 # If there is a proxy server statement in-use in the Host server, copy it across
@@ -524,6 +580,20 @@ if [ -n "$EXTRA_PACKAGES" ]; then
     EXTRA_PACKAGES_INSTALL="apt-get --assume-yes install ${EXTRA_PACKAGES//,/ }"
 fi
 
+if [ -f /etc/timezone ]; then
+    tz=$(cat /etc/timezone)
+    zonefile="$VROOTDIR/$VHOST/usr/share/zoneinfo/$tz"
+    if [ -f "$zonefile" ]; then
+	echo "$tz" >$VROOTDIR/$VHOST/etc/timezone
+	cp "$zonefile" "$VROOTDIR/$VHOST/etc/localtime"
+    fi
+fi
+
+if [ -n "$DEBCONF_FILE_DB" ]; then
+    cp "$DEBCONF_FILE_DB" "$VROOTDIR/$VHOST/config.dat"
+    SET_DEBCONF_OVERRIDE='export DEBCONF_DB_OVERRIDE="File{/config.dat}"'
+fi
+
 # ------------------------------------------------------------
 # From here on we do things live in the server
 # Generate the script that runs the rest of the setup from within the
@@ -531,19 +601,23 @@ fi
 cat << EOF > $VROOTDIR/$VHOST/vserver-config.sh
 #!/bin/sh
 
+$SET_DEBCONF_OVERRIDE
+
 [ -x /usr/sbin/locale-gen ] && /usr/sbin/locale-gen
 
 dselect update
 
-# tzsetup was part of base-config which is gone since etch
-# tzconfig is part of libc, so it should be ubiquitious
-if [ -x /usr/sbin/tzsetup ]; then
-    /usr/sbin/tzsetup -y
-elif [ -x /usr/sbin/tzconfig ]; then
-    /usr/sbin/tzconfig
+if [ ! -f /etc/timezone ]; then
+    # tzsetup was part of base-config which is gone since etch
+    # tzconfig is part of libc, so it should be ubiquitious
+    if [ -x /usr/sbin/tzsetup ]; then
+	/usr/sbin/tzsetup -y
+    elif [ -x /usr/sbin/tzconfig ]; then
+	/usr/sbin/tzconfig
+    fi
 fi
 
-dpkg-reconfigure passwd
+dpkg-reconfigure -u passwd
 
 tasksel
 
@@ -566,6 +640,8 @@ EOF
 chmod 755 $VROOTDIR/$VHOST/vserver-config.sh
 vserver $VHOST exec /vserver-config.sh
 rm -f $VROOTDIR/$VHOST/vserver-config.sh
+rm -f "$VROOTDIR/$VHOST/config.dat"
+rm -f "$VROOTDIR/$VHOST/config.dat-old"
 
 # If you need to install your SSH management keys into the vserver
 if [ -f "$SSH_KEYS" ]; then
